/*
***********************************************************************
*
* Copyright (C) 1999 Ericsson Radio Systems AB
* 
* The Association of Radio Industries and Businesses (ARIB) has the
* licence to reproduce, distribute and publish the Bit Exact C Code
* programs in this CD-ROM together with the other parts of the RCR
* STD-27 (Personal Digital Cellular Telecommunication System)
* specification.
* 
* Ericsson has declared Option 1 (royalty-free licence) to ARIB for
* its Essential Industrial Property Rights related to the ACELP
* specification and the Bit Exact C Code programs. The royalty-free
* licence is limited to the development and manufacturing of
* products complying with the RCR STD-27H specification (or
* subsequent versions) for sale and use in Japan. Copying of the
* program(s) for any other purpose requires the prior written
* permission from Ericsson.
* 
* The user of the Bit Exact C Code hereby acknowledges the
* following:
* 
* 1.  Ericsson does not provide the user with any warranties,
*     whether express or implied, including but not limited to
*     warranties of merchantability and fitness for a particular
*     purpose relating to the Bit Exact C Code.
* 
* 2.  Ericsson shall have no responsibility whatsoever to indemnify
*     the user against actions or claims for infringement of any
*     intellectual property rights, e.g. patents, copyrights,
*     registered designs or any other rights, by reason of the
*     user's use of the Bit Exact C Code.
* 
***********************************************************************
*/

/*************************************************************************
 *
 *   FUNCTION:  Refl_A()
 *
 *   PURPOSE: Convert from reflection coefficients to direct form coefficients
 *
 *   DESCRIPTION:
 *       reflection coefficients Q14 are converted to
 *       Directform coeffs in Q13
 *
 *************************************************************************/
#include "typedef.h"
#include "basic_op.h"
#include "cnst.h"


void Refl_A(
    Word16 refl[],  /* Q15 */
    Word16 a[]     /* Q12 */
)
{
    /* local variables */
    Word16 i,k;
    Word16 state[M];


    /* backward Levinson recursion */
    for (i = 0; i < M; i++) {
        a[i] = shr(refl[i],3); /* -> Q12 */
        for (k = 0; k < i; k++)
            state[k] = add(a[k],mult(refl[i],a[i-k-1]));
        for (k = 0; k < i; k++)
            a[k] = state[k];
    }
    return;
}


